/**************************************************************************
*  This file is part of the OSAL project (OS Abstraction Layer)
*
*  Copyright (c) 2020-2022 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
**************************************************************************/
#if !defined(__OSAL_H__)
#define __OSAL_H__

/**************************************************************************
*  Includes
**************************************************************************/
#include <stdint.h>

/**************************************************************************
*  All Structures and Common Constants
**************************************************************************/

#if defined(RTOS_TCTS)
/***********************************************/
/*                     TCTS                    */
/***********************************************/
#include "tcts.h"

#elif defined(RTOS_UCOS3)
/***********************************************/
/*              Micrium uC/OS-III              */
/***********************************************/
#include "source\os.h"
#include "lib_math.h"

#define OS_NAME         OS_NAME_STRING
#define OS_VER_STRING   OS_VERSION_STRING


#if (OS_CFG_TICK_RATE_HZ != 1000)
   Error: OS_CFG_TICK_RATE_HZ must be set to 1000;
#endif


#define OS_PRIO_MAX           (OS_CFG_PRIO_MAX-4)


/*
 * Default ticks per second
 */
#if !defined(OS_TICKS_PER_SECOND)
#define OS_TICKS_PER_SECOND   OS_CFG_TICK_RATE_HZ
#endif 


/*
 * This is the macro for defining the stack, use it like:
 *
 * static OS_TASK(MyStack, 512);
 *
 * The stack will be 8 byte aligned and the size 4 byte.
 */
#define OS_STACK(_n,_s)       uint8_t _n[((_s+3)& ~3)] __attribute__((aligned(0x8)))


/*
 * Some time management macros for converting MS => Ticks and Ticks => MS
 */
#if (OS_TICKS_PER_SECOND != 1000) 
#define OS_MS_2_TICKS(_a)     ((_a * OS_TICKS_PER_SECOND) / 1000)
#define OS_TICKS_2_MS(_a)     ((1000 / OS_TICKS_PER_SECOND) * _a)
#else
#define OS_MS_2_TICKS(_a)     (_a)
#define OS_TICKS_2_MS(_a)     (_a)
#endif


/*
 * Macro to check timeout
 *
 *    a = actual time
 *    s = start time
 *    t = timeout
 */
#define OS_TEST_TIMEOUT(_a,_s,_t)   (((uint32_t)(_a) - (uint32_t)(_s)) >= (uint32_t)(_t))


/*
 * Return code of OS functions
 */
#define OS_RC_OK               0
#define OS_RC_ERROR            -1
#define OS_RC_TIMEOUT          -2
#define OS_RC_NO_SPACE         -3
#define OS_RC_DELETED          -4
#define OS_RC_RESETED          -5


#define OS_WAIT_INFINITE      ((uint32_t)-1)
#define OS_SEMA_COUNTER_MAX   INT32_MAX


typedef OS_SEM                OS_SEMA;
typedef OS_FLAG_GRP           OS_EVENT;
typedef OS_Q                  OS_MBOX;

typedef void (*OS_TASK)(void *arg);


/*
 * Event
 */ 
typedef enum
{
   OS_EVENT_MODE_UNKNOWN = 0,
   OS_EVENT_MODE_OR,
   OS_EVENT_MODE_AND
} os_event_mode_t;


/*
 * Some task macros
 */
#define OS_TEST_STATE_NOT_IN_USED(_a)  (OS_TASK_STATE_DEL == ((OS_TCB*)_a)->TaskState) 


#else
   Error: No OS defined;
#endif

/**************************************************************************
*  Macro Definitions
**************************************************************************/

/*
 * Some resource macros
 */
#define OS_RES_CREATE(_a)     OS_SemaCreate(_a, 1, 1)
#define OS_RES_LOCK(_a)       OS_SemaWait(_a, OS_WAIT_INFINITE)
#define OS_RES_FREE(_a)       OS_SemaSignal(_a)

/**************************************************************************
*  Functions Definitions
**************************************************************************/

/*
 * General functionality
 */
void      OS_Init (void);
void      OS_Start (void);
void      OS_SysTickStart (void);

void      OS_OutputRuntimeStackInfo (void);
void      OS_OutputTaskInfo (void);
uint32_t  OS_GetStackInfo (OS_TCB *pTCB, uint32_t *pSize);


/*
 * Statistic functionality
 */
void      OS_StatEnable (void);
uint8_t   OS_StatGetCPULoad (void);
//uint32_t  OS_StatGetTotalTaskTime (void);


/*
 * Task functionality
 */
void      OS_TaskCreate (OS_TCB *pTCB, OS_TASK Task, void *pParam, int nPrio,
                         uint8_t *pStack, uint16_t wStackSize, char *pName);
//int       OS_TaskGetPriority (void);
int       OS_TaskChangePriority (int nPrio);
void      OS_TaskYield (void);
//void      OS_TaskExit (void);
//void      OS_TaskTerminateRequest (OS_TCB *pTCB);
//uint8_t   OS_TaskShouldTerminate (void);
//OS_TCB   *OS_TaskGetList (void);
void      OS_TaskSetStateNotInUsed (OS_TCB *pTCB);


/*
 * Time functionality
 */
void      OS_TimerCallback (void);
//void      OS_TimerSet (OS_TIMER_FUNC Timer, uint32_t dTimeoutMs);

uint32_t  OS_TimeGet (void);
uint32_t  OS_TimeGetSeconds (void);
void      OS_TimeDly (uint32_t dTimeoutMs);
//void      OS_TimeDlyUntil (uint32_t dTicks);
//void      OS_TimeWait (uint32_t dTimeoutMs);
uint32_t  OS_UnixtimeGet (void);
void      OS_UnixtimeSet (uint32_t dTime);
void      OS_TimezoneIDSet (int16_t wID);
int16_t   OS_TimezoneIDGet (void);
void      OS_TimezoneMinSet (int16_t wOffset);
int16_t   OS_TimezoneMinGet (void);
int32_t   OS_TimezoneSecGet (void);
void      OS_TimezoneDstSet (int8_t bOffset);
int32_t   OS_TimezoneDstSecGet (void);


/*
 * Semaphore functionality
 */
void      OS_SemaCreate (OS_SEMA *pSema, int32_t nCounterStart, int32_t nCounterMax);
//void      OS_SemaReset (OS_SEMA *pSema, int32_t nCounterStart);
void      OS_SemaDelete (OS_SEMA *pSema);
int       OS_SemaSignal (OS_SEMA *pSema);
int       OS_SemaSignalFromInt (OS_SEMA *pSema);
//int       OS_SemaSignalNoSched (OS_SEMA *pSema);
int       OS_SemaWait (OS_SEMA *pSema, uint32_t dTimeoutMs);


/*
 * Mutex functionality
 */
void      OS_MutexCreate (OS_MUTEX *pMutex);
void      OS_MutexSignal (OS_MUTEX *pMutex);
void      OS_MutexSignalFromInt (OS_MUTEX *pMutex);
int       OS_MutexWait (OS_MUTEX *pMutex, uint32_t dTimeoutMs);


/*
 * Event functionality
 */ 
void      OS_EventCreate (OS_EVENT *pEvent);
void      OS_EventDelete (OS_EVENT *pEvent);
void      OS_EventSet (OS_EVENT *pEvent, uint32_t dPattern);
void      OS_EventClr (OS_EVENT *pEvent, uint32_t dPattern);
void      OS_EventSetFromInt (OS_EVENT *pEvent, uint32_t dPattern);
int       OS_EventWait (OS_EVENT *pEvent, uint32_t dWaitPattern, 
                        os_event_mode_t Mode, uint32_t *pPattern, uint32_t dTimeoutMs);


/*
 * Mailbox functionality
 */       
void      OS_MboxCreate (OS_MBOX *pMbox, void **pBuffer, uint16_t wCounterMax);
//void      OS_MboxDelete (OS_MBOX *pMbox);
int       OS_MboxPostFromInt (OS_MBOX *pMbox, void *pMsg);
int       OS_MboxWait (OS_MBOX *pMbox, void **pMsg, uint32_t dTimeoutMs);

#endif /* !__OSAL_H__ */

/*** EOF ***/
