/**************************************************************************
*  This file is part of the OSAL project (OS Abstraction Layer)
*
*  Copyright (c) 2020 - 2022 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
**************************************************************************/
#define __OSAL_UCOS3_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdio.h>

#include "tal.h"
#include "osal.h"

#if defined (RTOS_UCOS3)

#include "lib_mem.h"

#if (OS_CFG_DBG_EN != 1)
error: OS_CFG_DBG_EN must be set to 1;
#endif

/*=======================================================================*/
/*  All extern data                                                      */
/*=======================================================================*/

#if defined(__ARM_ARCH_7EM__) || defined(__ARM_ARCH_7M__)

extern uint8_t __stack_start__[];
extern uint8_t __stack_end__[];

extern uint8_t __stack_process_start__[];
extern uint8_t __stack_process_end__[];

#endif

#if defined(__ARM_ARCH_7A__ )

/* Stack */
extern uint8_t __stack_start__[];
extern uint8_t __stack_end__[];

/* IRQ */
extern uint8_t __stack_irq_start__[];
extern uint8_t __stack_irq_end__[];

/* FIQ */
extern uint8_t __stack_fiq_start__[];
extern uint8_t __stack_fiq_end__[];

/* Supervisor */
extern uint8_t __stack_svc_start__[];
extern uint8_t __stack_svc_end__[];

/* Abort */
extern uint8_t __stack_abt_start__[];
extern uint8_t __stack_abt_end__[];

/* Undefined */
extern uint8_t __stack_und_start__[];
extern uint8_t __stack_und_end__[];

#endif

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*
 * Statistic stack size
 */
#if !defined(OS_STAT_STACK_SIZE)
#define OS_STAT_STACK_SIZE    512
#endif

/*
 * Idle, statistic and timer task priority
 */
#define IDLE_TASK_PRIO        (OS_CFG_PRIO_MAX - 1)
#define STAT_TASK_PRIO        (OS_CFG_PRIO_MAX - 2)
#define TIMER_TASK_PRIO       (OS_CFG_PRIO_MAX - 3)

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*
 * Ide, Statistic and SW-Watchdog task variables
 */
static OS_STACK(StatStack, OS_STAT_STACK_SIZE);
static OS_TCB TCBStat;

/*
 * Time variables
 */
static volatile uint32_t dSecTick    = 0;
static volatile uint32_t dUnixtime   = 0;

static int16_t   wTimezoneID           = 0;
static int16_t   wTimezoneOffsetMin    = 0;
static int32_t   nTimezoneOffsetSec    = 0;
static int32_t   dTimezoneDstOffsetSec = 0;

/*
 * Some statistics variables
 */
static uint8_t   bStatEnabled       = 0; 
static uint8_t   bStatCPULoad       = 0;
static uint32_t  dStatTotalTaskTime = 0;
static uint32_t  dStatHirResPeriod  = 0;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  GetIRQStackStart                                                     */
/*                                                                       */
/*  Return the IRQ stack start address.                                  */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: stack start address                                          */
/*************************************************************************/
static uint8_t *GetIRQStackStart (void)
{
   return((uint8_t*)OSCfg_ISRStkBasePtr);
} /* GetIRQStackStart */

/*************************************************************************/
/*  GetIRQStackEnd                                                       */
/*                                                                       */
/*  Return the IRQ stack end address.                                    */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: stack start address                                          */
/*************************************************************************/
static uint8_t *GetIRQStackEnd (void)
{
   return((uint8_t*)OSCfg_ISRStkBasePtr + (OSCfg_ISRStkSize * sizeof(CPU_STK)));
} /* GetIRQStackEnd */

/*************************************************************************/
/*  GetStackFreeCount                                                    */
/*                                                                       */
/*  The stack was initialized with 0xCC.                                 */
/*  Unused entries contains still 0xCC.                                  */
/*                                                                       */
/*  In    : pStart, pEnd                                                 */
/*  Out   : none                                                         */
/*  Return: Free entries of the stack                                    */
/*************************************************************************/
static uint32_t GetStackFreeCount (uint8_t *pStart, uint8_t *pEnd)
{
   uint32_t dFreeCount = 0;
   
   while (pStart < pEnd)
   {
      if (0xCC == *pStart)
      {
         dFreeCount++;   
      }
      else
      {
         break;
      }
      pStart++;
   }
   
   return(dFreeCount);
} /* GetStackFreeCount */

/*************************************************************************/
/*  StatUpdateTime                                                       */
/*                                                                       */
/*  Update the statistic time information from a task.                   */
/*                                                                       */
/*  Note: Interrupts are disabled and must be disabled.                  */
/*                                                                       */
/*  In    : pTask                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void StatUpdateTime (OS_TCB *pTask)
{
   uint16_t wStartTick;
   uint16_t wEndTick;
   uint16_t wStartTimer;
   uint16_t wEndTimer;
   uint16_t wDeltaTick;
   uint16_t wDeltaTimer;
   
   wStartTimer = pTask->dStatStartTime & 0xFFFF;
   wStartTick  = pTask->dStatStartTime >> 16;
   wEndTimer   = pTask->dStatEndTime & 0xFFFF;
   wEndTick    = pTask->dStatEndTime >> 16;
   
   wDeltaTick = wEndTick - wStartTick;

   if (wEndTimer > wStartTimer)
   {
      wDeltaTimer = wEndTimer - wStartTimer;
   }
   else
   {
      wDeltaTimer = (uint16_t)(dStatHirResPeriod - wStartTimer) + wEndTimer;
   }
   
   pTask->dStatTotalTime += (wDeltaTick * dStatHirResPeriod) + wDeltaTimer;
       
} /* StatUpdateTime */

/*************************************************************************/
/*  TaskSwHook                                                           */
/*                                                                       */
/*  This function is called when a task switch is performed.             */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void TaskSwHook (void)
{
   if (bStatEnabled)
   {
      /* Get task end time of the actual running task */
      OSTCBCurPtr->dStatEndTime = tal_CPUStatGetHiResCnt();
      StatUpdateTime(OSTCBCurPtr);

      /* Get task start time for next running task */
      OSTCBHighRdyPtr->dStatStartTime = tal_CPUStatGetHiResCnt();
   }
   
} /* TaskSwHook */

/*************************************************************************/
/*  StatTask                                                             */
/*                                                                       */
/*  In    : Task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void StatTask (void *pParam)
{   
   OS_TCB     *pTask;
   uint32_t    dIdleTime = 0;
   uint32_t    dStatUsage;

   CPU_SR_ALLOC();
   
   (void)pParam;
   
   bStatEnabled      = 1;
   dStatHirResPeriod = tal_CPUStatGetHiResPeriod();
   
   while (1)
   {
      if (bStatEnabled)
      {
         /* Get total task time */
         dStatTotalTaskTime = 0;
      
         /* Collect tota task time first */
         CPU_CRITICAL_ENTER();   /*lint !e717*/
         pTask = OSTaskDbgListPtr;
         while (pTask != NULL)
         {
            /* Count total TaskTime */
            dStatTotalTaskTime += pTask->dStatTotalTime;

            /* Find Idle time */
            if (IDLE_TASK_PRIO == pTask->Prio)
            {
               dIdleTime = pTask->dStatTotalTime;
            }
         
            pTask->dStatTotalLastTime = pTask->dStatTotalTime;
            pTask->dStatTotalTime     = 0;

            pTask = pTask->DbgNextPtr;
         }


         if (0 == dStatTotalTaskTime) dStatTotalTaskTime = 1;

         /* Calculate usage in percent */
         pTask = OSTaskDbgListPtr;
         while (pTask != NULL)
         {
            dStatUsage = (uint32_t)(((uint64_t)10000 * pTask->dStatTotalLastTime) / dStatTotalTaskTime);
            if (dStatUsage > 10000)
            {
               dStatUsage = 10000;
            }
            pTask->dStatUsage = dStatUsage;
            
            pTask = pTask->DbgNextPtr;
         }

         CPU_CRITICAL_EXIT();    /*lint !e717*/

         /*
          * Calculate CPU load like:
          * 
          *   bStatCPULoad = 100 - (uint8_t)((dIdleTime * 100) / dStatTotalTaskTime)
          *
          * But this makes problems with big counts. Therefore 
          * dIdleTime will not multiplied instead dStatTotalTaskTime
          * will be divided by 100.
          *
          *   bStatCPULoad = 100 - (uint8_t)(dIdleTime / (dStatTotalTaskTime / 100))
          */
         if (dStatTotalTaskTime > 100) /* Prevent division by zero */
         {
            bStatCPULoad = 100 - (uint8_t)(dIdleTime / (dStatTotalTaskTime / 100));
         }   
         else
         {
            bStatCPULoad = 1;
         }
      }         
      
      OS_TimeDly(OS_MS_2_TICKS(500));
   }
   
} /* StatTask */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  OS_Init                                                              */
/*                                                                       */
/*  Init the Operation System.                                           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_Init (void)
{
   OS_ERR err;
   
   tal_CPUInit();

   CPU_Init();    /* Initialize the uC/CPU services     */
   Mem_Init();    /* Initialize Memory Managment Module */
   CPU_IntDis();  /* Disable all Interrupts.            */

   OSInit(&err);  /* Init uC/OS-III. */
   (void)err;
   
   OS_AppTimeTickHookPtr = OS_TimerCallback;
   OS_AppTaskSwHookPtr   = TaskSwHook;
   
} /* OS_Init */

/*************************************************************************/
/*  OS_Start                                                             */
/*                                                                       */
/*  Start the "Cooperative Task Scheduler".                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
void OS_Start (void)
{
   OS_ERR err;

   OSStart(&err);
   (void)err;
   
} /* OS_Start */

/*************************************************************************/
/*  OS_SysTickStart                                                      */
/*                                                                       */
/*  Start the SysTick.                                                   */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
void OS_SysTickStart (void)
{
   tal_CPUSysTickStart();
   
} /* OS_SysTickStart */

/*************************************************************************/
/*  OS_OutputRuntimeStackInfo                                            */
/*                                                                       */
/*  Output the runtime stack information.                                */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_OutputRuntimeStackInfo (void)
{
   uint32_t dSize;
   uint32_t dFree;
   
#if defined(__ARM_ARCH_7EM__) || defined(__ARM_ARCH_7M__)

   TAL_PRINTF("*** Runtime Stack Info ***\n");
   TAL_PRINTF("\n");
   TAL_PRINTF("Stack          Size   Used   Free\n");
   TAL_PRINTF("===================================\n");
   
   dSize = (uint32_t)(__stack_end__ - __stack_start__);
   dFree = GetStackFreeCount(__stack_start__, __stack_end__);
   TAL_PRINTF("Main Stack     %4d   %4d   %4d\n", dSize, (dSize - dFree), dFree);
   
   dSize = (uint32_t)(__stack_process_end__ - __stack_process_start__);
   dFree = GetStackFreeCount(__stack_process_start__, __stack_process_end__);
   TAL_PRINTF("Process Stack  %4d   %4d   %4d\n", dSize, (dSize - dFree), dFree);
   
#endif   

#if defined(__ARM_ARCH_7A__ )

   TAL_PRINTF("*** Runtime Stack Info ***\n");
   TAL_PRINTF("\n");
   TAL_PRINTF("Stack              Size   Used   Free\n");
   TAL_PRINTF("=======================================\n");

   dSize = (uint32_t)(__stack_end__ - __stack_start__);
   dFree = GetStackFreeCount(__stack_start__, __stack_end__);
   TAL_PRINTF("User/System Stack  %4d   %4d   %4d\n", dSize, (dSize - dFree), dFree);
   
   dSize = (uint32_t)(__stack_irq_end__ - __stack_irq_start__);
   dFree = GetStackFreeCount(__stack_irq_start__, __stack_irq_end__);
   TAL_PRINTF("IRQ Stack          %4d   %4d   %4d\n", dSize, (dSize - dFree), dFree);

   dSize = (uint32_t)(__stack_fiq_end__ - __stack_fiq_start__);
   dFree = GetStackFreeCount(__stack_fiq_start__, __stack_fiq_end__);
   TAL_PRINTF("FIQ Stack          %4d   %4d   %4d\n", dSize, (dSize - dFree), dFree);

   dSize = (uint32_t)(__stack_svc_end__ - __stack_svc_start__);
   dFree = GetStackFreeCount(__stack_svc_start__, __stack_svc_end__);
   TAL_PRINTF("Supervisior Stack  %4d   %4d   %4d\n", dSize, (dSize - dFree), dFree);

   dSize = (uint32_t)(__stack_abt_end__ - __stack_abt_start__);
   dFree = GetStackFreeCount(__stack_abt_start__, __stack_abt_end__);
   TAL_PRINTF("Abort Stack        %4d   %4d   %4d\n", dSize, (dSize - dFree), dFree);

   dSize = (uint32_t)(__stack_und_end__ - __stack_und_start__);
   dFree = GetStackFreeCount(__stack_und_start__, __stack_und_end__);
   TAL_PRINTF("Undefined Stack    %4d   %4d   %4d\n", dSize, (dSize - dFree), dFree);

#endif
   
   (void)dSize;
   (void)dFree;
   
   TAL_PRINTF("\n");
} /* OS_OutputRuntimeStackInfo */

/*************************************************************************/
/*  OS_OutputTaskInfo                                                    */
/*                                                                       */
/*  Output the Stack and Time information from the tasks.                */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_OutputTaskInfo (void)
{
   OS_TCB   *pTCB;
   uint8_t  *pStart;
   uint8_t  *pEnd;
   uint32_t  dSize;
   uint32_t  dFree;
   uint16_t  wTime1 = 0;
   uint16_t  wTime2 = 0;

   CPU_SR_ALLOC();
   
   TAL_PRINTF("*** Task Info ***\n");
   TAL_PRINTF("\n");
   TAL_PRINTF("Task              Prio   Size   Used   Free   CPU %%\n");
   TAL_PRINTF("====================================================\n");

   /* Display "IRQ" Stack */
   pStart = GetIRQStackStart();
   pEnd   = GetIRQStackEnd();
   dSize  = (uint32_t)pEnd - (uint32_t)pStart;
   dFree  = GetStackFreeCount(pStart, pEnd);
   
   TAL_PRINTF("%-16s  ", "- IRQ -");
   TAL_PRINTF("  --   %4d   %4d   %4d   -----\n", dSize, (dSize - dFree), dFree);

   /* Get start of task list */
   CPU_CRITICAL_ENTER();   /*lint !e717*/
   pTCB = OSTaskDbgListPtr;
   CPU_CRITICAL_EXIT();    /*lint !e717*/
   
   while (pTCB != NULL)
   {
      if( (pTCB->Prio != TIMER_TASK_PRIO) &&
          (pTCB->Prio != STAT_TASK_PRIO)  &&
          (pTCB->Prio != IDLE_TASK_PRIO)  )
      {          
         if (pTCB->NamePtr[0] != 0)
         {
            TAL_PRINTF("%-16s  ", pTCB->NamePtr);
         }
         else
         {
            TAL_PRINTF("%-16s  ", "----------------");
         }
              
         dFree  = OS_GetStackInfo (pTCB, &dSize);
         wTime1 = (uint16_t)(pTCB->dStatUsage / 100);
         wTime2 = (uint16_t)(pTCB->dStatUsage % 100);

         TAL_PRINTF("%4d   %4d   %4d   %4d   %2d.%02d\n", pTCB->Prio, dSize, (dSize - dFree), dFree, wTime1, wTime2);
      }            
      
      CPU_CRITICAL_ENTER();   /*lint !e717*/
      pTCB = pTCB->DbgNextPtr;
      CPU_CRITICAL_EXIT();    /*lint !e717*/
   }   
      
   /* 
    * Display Timer task info
    */
#if (OS_CFG_TMR_EN > 0u)
   pTCB   = &OSTmrTaskTCB;
   dFree  = OS_GetStackInfo(pTCB, &dSize);
   wTime1 = (uint16_t)(pTCB->dStatUsage / 100);
   wTime2 = (uint16_t)(pTCB->dStatUsage % 100);

   TAL_PRINTF("%-16s  ", pTCB->NamePtr);
   TAL_PRINTF("%4d   %4d   %4d   %4d   %2d.%02d\n", pTCB->Prio, dSize, (dSize - dFree), dFree, wTime1, wTime2);
#endif

   /* 
    * Display Statistic task info
    */
   if (bStatEnabled)
   {
      pTCB   = &TCBStat;
      dFree  = OS_GetStackInfo(pTCB, &dSize);
      wTime1 = (uint16_t)(pTCB->dStatUsage / 100);
      wTime2 = (uint16_t)(pTCB->dStatUsage % 100);

      TAL_PRINTF("%-16s  ", pTCB->NamePtr);
      TAL_PRINTF("%4d   %4d   %4d   %4d   %2d.%02d\n", pTCB->Prio, dSize, (dSize - dFree), dFree, wTime1, wTime2);
   }      

   /* 
    * Display Idle task info
    */
   pTCB   = &OSIdleTaskTCB;
   dFree  = OS_GetStackInfo(pTCB, &dSize);
   wTime1 = (uint16_t)(pTCB->dStatUsage / 100);
   wTime2 = (uint16_t)(pTCB->dStatUsage % 100);

   TAL_PRINTF("%-16s  ", pTCB->NamePtr);
   TAL_PRINTF("%4d   %4d   %4d   %4d   %2d.%02d\n", pTCB->Prio, dSize, (dSize - dFree), dFree, wTime1, wTime2);

   TAL_PRINTF("\n--- CPU load: %d%% ---\n", OS_StatGetCPULoad());

   TAL_PRINTF("\n");   

} /* OS_OutputTaskInfo */

/*************************************************************************/
/*  OS_GetStackInfo                                                      */
/*                                                                       */
/*  Return the info about the stack.                                     */
/*                                                                       */
/*  In    : pTCB, pSize                                                  */
/*  Out   : pSize                                                        */
/*  Return: dFree                                                        */
/*************************************************************************/
uint32_t OS_GetStackInfo (OS_TCB *pTCB, uint32_t *pSize)
{
   uint8_t  *pStart;
   uint8_t  *pEnd;
   uint32_t  dSize;
   uint32_t  dFree;
   
   pStart = (uint8_t*)pTCB->StkBasePtr;
   pEnd   = (uint8_t*)pStart + (pTCB->StkSize * sizeof(CPU_STK));
   dSize  = (uint32_t)pEnd - (uint32_t)pStart;
   dFree  = GetStackFreeCount(pStart, pEnd);
   
   *pSize = dSize;

   return(dFree);
} /* OS_GetStackInfo */

/*************************************************************************/
/*  OS_StatEnable                                                        */
/*                                                                       */
/*  Enable the statistic functionality.                                  */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_StatEnable (void)
{
   /* Create the Statistic task */
   OS_TaskCreate(&TCBStat, StatTask, NULL, STAT_TASK_PRIO,
                 StatStack, sizeof(StatStack), "[StatTask]");

} /* OS_StatEnable */

/*************************************************************************/
/*  OS_StatGetCPULoad                                                    */
/*                                                                       */
/*  Return the CPU load in percent.                                      */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
uint8_t OS_StatGetCPULoad (void)
{
   return(bStatCPULoad);
} /* OS_StatGetCPULoad */

/*************************************************************************/
/*  OS_TaskCreate                                                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: TAL_OK / error cause                                         */
/*************************************************************************/
void OS_TaskCreate (OS_TCB *pTCB, OS_TASK Task, void *pParam, int nPrio,
                    uint8_t *pStack, uint16_t wStackSize, char *pName)
{
   OS_ERR err;

   OSTaskCreate(pTCB,
                pName,
                Task,
                pParam,
                (OS_PRIO)nPrio,
                (CPU_STK*)pStack,
                
                (wStackSize/10), /* Stack limit */
                wStackSize / sizeof(CPU_STK),
                
                0u,
                0u,
                0u,
                (OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
                &err);  /*lint !e826*/
   (void)err;                

} /* OS_TaskCreate */

/*************************************************************************/
/*  OS_TaskChangePriority                                                */
/*                                                                       */
/*  Change the priority of the actual running task.                      */
/*                                                                       */
/*  In    : Prio                                                         */
/*  Out   : none                                                         */
/*  Return: The old priority                                             */
/*************************************************************************/
int OS_TaskChangePriority (int nPrio)
{
   OS_ERR             err;
   OS_TCB *pTCB     = OSTCBCurPtr;
   int      OldPrio = pTCB->Prio;

   OSTaskChangePrio(pTCB, (OS_PRIO)nPrio, &err);
   (void)err;

   return(OldPrio);
} /* OS_TaskChangePriority */

/*************************************************************************/
/*  OS_TaskYield                                                         */
/*                                                                       */
/*  Give CPU time to the next task from the ReadyList.                   */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_TaskYield (void)
{
   OS_ERR err;
   
   OSTimeDly(2, OS_OPT_TIME_DLY, &err);
   (void)err;
   
} /* OS_TaskYield */

/*************************************************************************/
/*  OS_TaskSetStateNotInUsed                                             */
/*                                                                       */
/*  Change the task state to "not in used".                              */
/*                                                                       */
/*  In    : pTCB                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_TaskSetStateNotInUsed (OS_TCB *pTCB)
{
   if (pTCB != NULL)
   {
      pTCB->TaskState = OS_TASK_STATE_DEL;
   }
   
} /* OS_TaskSetNotInUsed */

/*************************************************************************/
/*  OS_TimerCallback                                                     */
/*                                                                       */
/*  This function will be called from the Systick interrupt.             */
/*                                                                       */
/*  Note: Interrupts are disabled and must be disabled.                  */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_TimerCallback (void)
{
   static uint32_t dTickCnt = 0;
   
   /* 
    * Second ticker
    */
   dTickCnt++;
   if (OS_TICKS_PER_SECOND == dTickCnt)
   {
      dTickCnt = 0;
      dSecTick++;
      dUnixtime++;
   }
   
} /* OS_TimerCallback */

/*************************************************************************/
/*  OS_TimeGet                                                           */
/*                                                                       */
/*  Return the time from the system ticker.                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Systick                                                      */
/*************************************************************************/
uint32_t OS_TimeGet (void)
{
   OS_ERR err;
   
   return( OSTimeGet(&err) );
} /* OS_TimeGet */

/*************************************************************************/
/*  OS_TimeGetSeconds                                                    */
/*                                                                       */
/*  Return the time from the second ticker.                              */
/*                                                                       */
/*  Out   : none                                                         */
/*  Return: dSecTick                                                     */
/*************************************************************************/
uint32_t OS_TimeGetSeconds (void)
{
   uint32_t dValue;

   CPU_SR_ALLOC();

   CPU_CRITICAL_ENTER();   /*lint !e717*/
   dValue = dSecTick;
   CPU_CRITICAL_EXIT();    /*lint !e717*/

   return(dValue);
} /* OS_TimeGetSeconds */

/*************************************************************************/
/*  OS_TimeDly                                                           */
/*                                                                       */
/*  Causes the current task to wait for a specified interval or, if the  */
/*  specified interval is zero, to give up the CPU for another task      */
/*  (with higher or same priority).                                      */
/*                                                                       */
/*  Note: Must not used from inside an interrupt.                        */
/*                                                                       */
/*  In    : TimeoutMs                                                    */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_TimeDly (uint32_t dTimeoutMs)
{
   OS_ERR err;
   
   OSTimeDly(OS_MS_2_TICKS(dTimeoutMs), OS_OPT_TIME_DLY, &err);
   (void)err;
   
} /* OS_TimeDly */

/*************************************************************************/
/*  OS_UnixtimeGet                                                       */
/*                                                                       */
/*  Return the unixtime.                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Unixtime                                                     */
/*************************************************************************/
uint32_t OS_UnixtimeGet (void)
{
   uint32_t dValue;

   CPU_SR_ALLOC();

   CPU_CRITICAL_ENTER();   /*lint !e717*/
   dValue = dUnixtime;
   CPU_CRITICAL_EXIT();    /*lint !e717*/

   return(dValue);
} /* OS_UnixtimeGet */

/*************************************************************************/
/*  OS_UnixtimeSet                                                       */
/*                                                                       */
/*  Set a new unixtime.                                                  */
/*                                                                       */
/*  In    : dTime                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_UnixtimeSet (uint32_t dTime)
{
   dUnixtime = dTime;
   
} /* OS_UnixtimeSet */

/*************************************************************************/
/*  OS_TimezoneIDSet                                                     */
/*                                                                       */
/*  Set the timezone ID.                                                 */
/*                                                                       */
/*  In    : wID                                                          */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_TimezoneIDSet (int16_t wID)
{
   wTimezoneID = wID;
   
} /* OS_TimezoneIDSet */

/*************************************************************************/
/*  OS_TimezoneIDGet                                                     */
/*                                                                       */
/*  Return the timezone ID.                                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Timezone                                                     */
/*************************************************************************/
int16_t OS_TimezoneIDGet (void)
{
   return(wTimezoneID);
} /* OS_TimezoneIDGet */

/*************************************************************************/
/*  OS_TimezoneMinSet                                                    */
/*                                                                       */
/*  Set the timezone offset to UTC in minutes.                           */
/*                                                                       */
/*  In    : nOffset                                                      */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_TimezoneMinSet (int16_t nOffset)
{
   wTimezoneOffsetMin = nOffset;
   nTimezoneOffsetSec = nOffset * 60;
   
} /* OS_TimezoneMinSet */

/*************************************************************************/
/*  OS_TimezoneMinGet                                                    */
/*                                                                       */
/*  Return the timezone offset in minutes to UTC.                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Timezone                                                     */
/*************************************************************************/
int16_t OS_TimezoneMinGet (void)
{
   return(wTimezoneOffsetMin);
} /* OS_TimezoneMinGet */

/*************************************************************************/
/*  OS_TimezoneSecGet                                                    */
/*                                                                       */
/*  Return the timezone offset in seconds to UTC.                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Timezone                                                     */
/*************************************************************************/
int32_t OS_TimezoneSecGet (void)
{
   return(nTimezoneOffsetSec);
} /* OS_TimezoneSecGet */

/*************************************************************************/
/*  OS_TimezoneDstSet                                                    */
/*                                                                       */
/*  Set the daylight saving time offset to UTC in hours.                 */
/*                                                                       */
/*  In    : nOffset                                                      */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_TimezoneDstSet (int8_t nOffset)
{
   dTimezoneDstOffsetSec = nOffset * 60 * 60;
   
} /* OS_TimezoneDstSet */

/*************************************************************************/
/*  OS_TimezoneDstSecGet                                                 */
/*                                                                       */
/*  Return the daylight saving time offset in seconds to UTC.            */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Timezone                                                     */
/*************************************************************************/
int32_t OS_TimezoneDstSecGet (void)
{
   return(dTimezoneDstOffsetSec);
} /* OS_TimezoneDstSecGet */

/*************************************************************************/
/*  OS_SemaCreate                                                        */
/*                                                                       */
/*  Create a new semaphore.                                              */
/*                                                                       */
/*  In    : pSema, dCounterStart, dCounterMax                            */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_SemaCreate (OS_SEMA *pSema, int32_t nCounterStart, int32_t nCounterMax)
{
   OS_ERR err;

   OSSemCreate(pSema, NULL, (OS_SEM_CTR)nCounterStart, &err);
   (void)err;
   (void)nCounterMax;
   
} /* OS_SemaCreate */

/*************************************************************************/
/*  OS_SemaDelete                                                        */
/*                                                                       */
/*  Delete a semaphore. All waiting tasks will be released.              */
/*                                                                       */
/*  In    : pSema                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_SemaDelete (OS_SEMA *pSema)
{
   OS_ERR err;

   OSSemDel(pSema, OS_OPT_DEL_ALWAYS, &err);
   (void)err;

} /* OS_SemaDelete */

/*************************************************************************/
/*  OS_SemaSignalFromInt                                                 */
/*                                                                       */
/*  Signal a semaphore from interrupt.                                   */
/*                                                                       */
/*  Must be called with interrupts disabled.                             */
/*                                                                       */
/*  Note: Interrupts are disabled and must be disabled.                  */
/*                                                                       */
/*  In    : pSema                                                        */
/*  Out   : none                                                         */
/*  Return: 1 = must schedule / 0 = schedule not needed / error cause    */
/*************************************************************************/
int OS_SemaSignalFromInt (OS_SEMA *pSema)
{
   OS_ERR err;

   OSSemPost(pSema, OS_OPT_POST_NO_SCHED, &err);
   (void)err;

   return(0);
} /* OS_SemaSignalFromInt */

/*************************************************************************/
/*  OS_SemaSignal                                                        */
/*                                                                       */
/*  Signal a semaphore.                                                  */
/*                                                                       */
/*  Note: Must not used from inside an interrupt.                        */
/*                                                                       */
/*  In    : pSema                                                        */
/*  Out   : none                                                         */
/*  Return: OS_RC_OK / error cause                                       */
/*************************************************************************/
int OS_SemaSignal (OS_SEMA *pSema)
{
   int    rc = OS_RC_OK;
   OS_ERR err;   
   
   OSSemPost(pSema, OS_OPT_POST_ALL, &err);
   if (err != OS_ERR_NONE)
   {
      rc = OS_RC_ERROR;
   }
   
   return(rc);
} /* OSSemaSignal */

/*************************************************************************/
/*  OS_SemaWait                                                          */
/*                                                                       */
/*  Blocks the task while waiting for the semaphore, with timeout.       */
/*                                                                       */
/*  In case the timeout value (dTimeoutMs) is 0, OSSemaWait will use     */
/*  polling to acquire the resource. If the resource counter (dCounter)  */
/*  is 0, the error cause is OS_RC_TIMEOUT.                              */
/*                                                                       */
/*  With a timeout value (dTimeoutMs) of OS_WAIT_INFINITE the waiting    */
/*  time is not ended until the semaphore is signaled.                   */
/*                                                                       */
/*  Note: Must not used from inside an interrupt.                        */
/*                                                                       */
/*  In    : pSema, dTimeoutMs                                            */
/*  Out   : none                                                         */
/*  Return: OS_RC_OK / error casue                                       */
/*************************************************************************/
int OS_SemaWait (OS_SEMA *pSema, uint32_t dTimeoutMs)
{
   int     rc = OS_RC_OK;
   OS_ERR  err;
   OS_TICK timeout;
   
   if (OS_WAIT_INFINITE == dTimeoutMs)
   {
      timeout = 0;
   }
   else
   {
      timeout = OS_MS_2_TICKS(dTimeoutMs);
   }
   
   OSSemPend(pSema, timeout, OS_OPT_PEND_BLOCKING, NULL, &err);
   if (OS_ERR_TIMEOUT == err)
   {
      rc = OS_RC_TIMEOUT; 
   }

   return(rc);   
} /* OS_SemaWait */

/*************************************************************************/
/*  OS_MutexCreate                                                       */
/*                                                                       */
/*  Create a new mutex.                                                  */
/*                                                                       */
/*  In    : pMutex                                                       */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_MutexCreate (OS_MUTEX *pMutex)
{
   OS_ERR err;
   
   OSMutexCreate(pMutex, NULL, &err);
   (void)err;
   
} /* OS_MutexCreate */

/*************************************************************************/
/*  OS_MutexSignalFromInt                                                */
/*                                                                       */
/*  Signal a mutex from interrupt.                                       */
/*                                                                       */
/*  Must be called with interrupts disabled.                             */
/*                                                                       */
/*  Note: Interrupts are disabled and must be disabled.                  */
/*                                                                       */
/*  In    : pMutex                                                       */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_MutexSignalFromInt (OS_MUTEX *pMutex)
{
   OS_ERR err;

   OSMutexPost(pMutex, OS_OPT_POST_NONE, &err);
   (void)err;
   
} /* OS_MutexSignalFromInt */

/*************************************************************************/
/*  OS_MutexSignal                                                       */
/*                                                                       */
/*  Signal a mutex.                                                      */
/*                                                                       */
/*  Note: Must not used from inside an interrupt.                        */
/*                                                                       */
/*  In    : pSema                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_MutexSignal (OS_MUTEX *pMutex)
{
   OS_ERR err;

   OSMutexPost(pMutex, OS_OPT_POST_NONE, &err);
   (void)err;
   
} /* OS_MutexSignal */

/*************************************************************************/
/*  OS_MutexWait                                                         */
/*                                                                       */
/*  Blocks the task while waiting for the mutex, with timeout.           */
/*                                                                       */
/*  In case the timeout value (dTimeoutMs) is 0, OSSemaWait will use     */
/*  polling to acquire the resource. If the resource counter (dCounter)  */
/*  is 0, the error cause is OS_RC_TIMEOUT.                              */
/*                                                                       */
/*  With a timeout value (dTimeoutMs) of OS_WAIT_INFINITE the waiting    */
/*  time is not ended until the semaphore is signaled.                   */
/*                                                                       */
/*  Note: Must not used from inside an interrupt.                        */
/*                                                                       */
/*  In    : pSema, dTimeoutMs                                            */
/*  Out   : none                                                         */
/*  Return: OS_RC_OK / error casue                                       */
/*************************************************************************/
int OS_MutexWait (OS_MUTEX *pMutex, uint32_t dTimeoutMs)
{
   int     rc;
   OS_ERR  err;
   OS_TICK timeout;
   
   if (OS_WAIT_INFINITE == dTimeoutMs)
   {
      timeout = 0;
   }
   else
   {
      timeout = OS_MS_2_TICKS(dTimeoutMs);
   }
   
   OSMutexPend(pMutex, timeout, OS_OPT_PEND_BLOCKING, NULL, &err);
   
   switch (err)
   {
      case OS_ERR_TIMEOUT: rc = OS_RC_TIMEOUT; break;
      case OS_ERR_NONE:    rc = OS_RC_OK;      break;
      default:             rc = OS_RC_ERROR;   break;
   }
   
   return(rc);
} /* OS_MutexWait */

/*************************************************************************/
/*  OS_EventCreate                                                       */
/*                                                                       */
/*  Create a new event.                                                  */
/*                                                                       */
/*  In    : pEvent                                                       */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_EventCreate (OS_EVENT *pEvent)
{
   OS_ERR err;

   OSFlagCreate(pEvent, NULL, 0, &err);
   (void)err;
   
} /* OS_EventCreate */

/*************************************************************************/
/*  OS_EventDelete                                                       */
/*                                                                       */
/*  Delete an event.                                                     */
/*                                                                       */
/*  In    : pEvent                                                       */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_EventDelete (OS_EVENT *pEvent)
{
   OS_ERR err;

   OSFlagDel(pEvent, OS_OPT_DEL_ALWAYS, &err);
   (void)err;
  
} /* OS_EventDelete */

/*************************************************************************/
/*  OS_EventSetFromInt                                                   */
/*                                                                       */
/*  Signal an event from interrupt.                                      */
/*                                                                       */
/*  Must be called with interrupts disabled.                             */
/*                                                                       */
/*  Note: Interrupts are disabled and must be disabled.                  */
/*                                                                       */
/*  In    : pEvent, dPattern                                             */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_EventSetFromInt (OS_EVENT *pEvent, uint32_t dPattern)
{
   OS_ERR err;
   
   OSFlagPost(pEvent, dPattern, OS_OPT_POST_FLAG_SET, &err);
   (void)err;
   
} /* OS_EventSetFromInt */

/*************************************************************************/
/*  OS_EventSet                                                          */
/*                                                                       */
/*  Signal an event.                                                     */
/*                                                                       */
/*  Note: Must not used from inside an interrupt.                        */
/*                                                                       */
/*  In    : pEvent, dPattern                                             */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_EventSet (OS_EVENT *pEvent, uint32_t dPattern)
{
   OS_ERR err;
   
   OSFlagPost(pEvent, dPattern, OS_OPT_POST_FLAG_SET, &err);
   (void)err;
   
} /* OS_EventSet */

/*************************************************************************/
/*  OS_EventClr                                                          */
/*                                                                       */
/*  Clear the event pattern.                                             */
/*                                                                       */
/*  Note: Must not used from inside an interrupt.                        */
/*                                                                       */
/*  In    : pEvent, dPattern                                             */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_EventClr (OS_EVENT *pEvent, uint32_t dPattern)
{
   OS_ERR err;
   
   OSFlagPost(pEvent, dPattern, OS_OPT_POST_FLAG_CLR, &err);
   (void)err;
   
} /* OS_EventClr */

/*************************************************************************/
/*  OS_EventWait                                                         */
/*                                                                       */
/*  Blocks the task while waiting for the event, with timeout.           */
/*                                                                       */
/*  With a timeout value (dTimeoutMs) of OS_WAIT_INFINITE the waiting    */
/*  time is not ended until the event is signaled.                       */
/*                                                                       */
/*  Note: Must not used from inside an interrupt.                        */
/*                                                                       */
/*  In    : pEvent, dWaitPattern, Mode, pPattern, dTimeoutMs             */
/*  Out   : none                                                         */
/*  Return: OS_RC_OK / error casue                                       */
/*************************************************************************/
int OS_EventWait (OS_EVENT *pEvent, uint32_t dWaitPattern, 
                  os_event_mode_t Mode, uint32_t *pPattern, uint32_t dTimeoutMs)
{
   OS_ERR    err;
   int       rc = OS_RC_OK;
   OS_TICK   timeout;
   OS_OPT    opt;
   uint32_t dValue;
   
   if (OS_WAIT_INFINITE == dTimeoutMs)
   {
      timeout = 0;
   }
   else
   {
      timeout = OS_MS_2_TICKS(dTimeoutMs);
   }
   
   switch (Mode)
   {
      case OS_EVENT_MODE_OR:  opt = OS_OPT_PEND_FLAG_SET_ANY; break;
      case OS_EVENT_MODE_AND: opt = OS_OPT_PEND_FLAG_SET_ALL; break;
      default:                opt = OS_OPT_PEND_FLAG_SET_ANY; break;
   }
   
   opt |= OS_OPT_PEND_FLAG_CONSUME; 

   *pPattern = 0;
   dValue = OSFlagPend(pEvent, dWaitPattern, timeout, opt, NULL, &err);
   if (OS_ERR_NONE == err)
   {
      *pPattern = dValue;
   }
   
   switch (err)
   {
      case OS_ERR_TIMEOUT: rc = OS_RC_TIMEOUT; break;
      case OS_ERR_NONE:    rc = OS_RC_OK;      break;
      default:             rc = OS_RC_ERROR;   break;
   }

   return(rc);
} /* OS_EventWait */

/*************************************************************************/
/*  OS_MboxCreate                                                        */
/*                                                                       */
/*  Create a new mailbox.                                                */
/*                                                                       */
/*  In    : pMbox, pBuffer, nCounterMax                                  */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OS_MboxCreate (OS_MBOX *pMbox, void **pBuffer, uint16_t wCounterMax)
{
   OS_ERR err;

   (void)pBuffer;
   
   OSQCreate(pMbox, NULL, wCounterMax, &err);
   (void)err;
   
} /* OS_MboxCreate */

/*************************************************************************/
/*  OS_MboxPostFromInt                                                   */
/*                                                                       */
/*  Tries to post a message to the mailbox.                              */
/*                                                                       */
/*  Must be called with interrupts disabled.                             */
/*                                                                       */
/*  Note: Interrupts are disabled and must be disabled.                  */
/*                                                                       */
/*  In    : pMbox, pMsg                                                  */
/*  Out   : none                                                         */
/*  Return: OS_RC_OK / OS_RC_NO_SPACE                                    */
/*************************************************************************/
int OS_MboxPostFromInt (OS_MBOX *pMbox, void *pMsg)
{
   int    rc = OS_RC_NO_SPACE;
   OS_ERR err;
   
   OSQPost(pMbox, pMsg, sizeof(void*), OS_OPT_POST_FIFO, &err);
   if (OS_ERR_NONE == err)
   {
      rc = OS_RC_OK;
   }

   return(rc);
} /* OS_MboxPostFromInt */

/*************************************************************************/
/*  OS_MboxWait                                                          */
/*                                                                       */
/*  Blocks the task while waiting for the mailbox, with timeout.         */
/*                                                                       */
/*  In case the timeout value (dTimeoutMs) is 0, OSMboxWait will use     */
/*  polling to acquire the message. If the resource counter (dCounter)   */
/*  is 0, the error cause is OS_RC_TIMEOUT.                              */
/*                                                                       */
/*  With a timeout value (dTimeoutMs) of OS_WAIT_INFINITE the waiting    */
/*  time is not ended until the mailbox is signaled.                     */
/*                                                                       */
/*  Note: Must not used from inside an interrupt.                        */
/*                                                                       */
/*  In    : pMbox, pMsg, dTimeoutMs                                      */
/*  Out   : none                                                         */
/*  Return: OS_RC_OK / OS_RC_TIMEOUT                                     */
/*************************************************************************/
int OS_MboxWait (OS_MBOX *pMbox, void **pMsg, uint32_t dTimeoutMs)
{
   int         rc = OS_RC_TIMEOUT;
   OS_ERR      err;   
   OS_MSG_SIZE size;
   OS_TICK     timeout;
   void       *tmp;
   
   if (0 == dTimeoutMs)
   {
      tmp = OSQPend(pMbox, 0, OS_OPT_PEND_NON_BLOCKING, &size, NULL, &err);
      if (OS_ERR_NONE == err)
      {
         rc = OS_RC_OK;
         *pMsg = tmp;         
      }
   }
   else
   {
      if (OS_WAIT_INFINITE == dTimeoutMs)
      {
         timeout = 0;   
      }
      else
      {
         timeout = OS_MS_2_TICKS(dTimeoutMs);
      }

      tmp = OSQPend(pMbox, timeout, OS_OPT_PEND_BLOCKING, &size, NULL, &err);
      if (OS_ERR_NONE == err)
      {
         rc = OS_RC_OK;
         *pMsg = tmp;         
      }
   }

   (void)size;

   return(rc);
} /* OS_MboxWait*/

#endif /* #if defined (RTOS_UCOS3) */

/*** EOF ***/


