/**************************************************************************
*  Copyright (c) 2022 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdio.h>
#include "tal.h"
#include "terminal.h"
#include "xmempool.h"
#include "ipstack.h"
#include "iperf.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define LED_MODE_WAIT   0
#define LED_MODE_READY  1

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/* 
 * Some TASK variables like stack and task control block.
 */
static OS_STACK (StartStack,  TASK_START_STK_SIZE);
static OS_STACK (LEDStack,    TASK_LED_STK_SIZE);

static OS_TCB   TCBStartTask;
static OS_TCB   TCBLed;

static int nDHCPCallbackBound = 0;
static int nLedMode = LED_MODE_WAIT; 


static int nIPDhcpIsUsed = 0;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  _OSInit                                                              */
/*                                                                       */
/*  In the case of a uC/OS project, "OS_Init" must be called first.      */
/*  Whereas in a TCTS project "tal_Init" must be called first.           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void _OSInit (void)
{
#if defined(RTOS_UCOS3)
   OS_Init();     /* Init operating system */
#endif

#if defined(RTOS_TCTS)
   tal_Init();    /* Init the "Tiny Abstraction Layer" */
   xmem_Init();   /* Initialize the memory pool */      
#endif
   
} /* _OSInit */

/*************************************************************************/
/*  _OSInitPost                                                          */
/*                                                                       */
/*  In the case of a uC/OS project, tal_init is not called first, but    */
/*  later in the StartTask.                                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void _OSInitPost (void)
{
#if defined(RTOS_UCOS3)
   tal_Init();    /* Init the "Tiny Abstraction Layer" */
   xmem_Init();   /* Initialize the memory pool */      
#endif

#if defined(RTOS_TCTS)
   /* Do nothing */
#endif
   
} /* _OSInitPost */

/*************************************************************************/
/*  DHCPCallbackBound                                                    */
/*                                                                       */
/*  Callback is called whenever the DHCP is bound.                       */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void DHCPCallbackBound (void)
{
   nDHCPCallbackBound = 1;
   
} /* DHCPCallbackBound */

/*************************************************************************/
/*  SecFunction                                                          */
/*                                                                       */
/*  This function wil be called each second.                             */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void SecFunction (void)
{
   static int nOldMode = 1;
   static int nNewMode = 1;
   
   if (1 == nDHCPCallbackBound)
   {
      nDHCPCallbackBound = 0;
            
      nLedMode = LED_MODE_READY;;
   }
   
   /********************************************/   
   
   nNewMode = IP_IF_IsReady(IFACE_ANY);
   if (nOldMode != nNewMode)
   {
      /* Check for lost */
      if ((1 == nOldMode) && (0 == nNewMode))
      {
         nLedMode = LED_MODE_WAIT;
         IP_DHCP_Stop(0);
      }
        
      /* Check for available */
      if ((0 == nOldMode) && (1 == nNewMode))
      {
         if (nIPDhcpIsUsed != 0)
         {
            /* LED mode will be set in DHCPCallbackBound */
            IP_DHCP_CallbackSet(0, DHCPCallbackBound);   
            IP_DHCP_Start(0);
         }
         else
         {
            nLedMode = LED_MODE_READY;
         }
      }
  
      nOldMode = nNewMode;
   }
   
} /* SecFunction */

/*************************************************************************/
/*  EthernetInit                                                         */
/*                                                                       */
/*  Initialize the ethernet interface.                                   */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void EthernetInit (void)
{
   uint8_t  MACAddr[6];
   
   IP_Init();
   
   /* Configure the MAC address */
   tal_BoardGetMACAddress(MACAddr);

   /* Configure the IP interface */
   IP_IF_MACSet(0, MACAddr);
   IP_IF_AddrSet(0, atoh("192.168.1.200") );
   IP_IF_MaskSet(0, atoh("255.255.255.0") );
   IP_IF_GWSet(0,   atoh("0.0.0.0") );
   
   /* Start the IP stack */
   IP_IF_Start(0);

   /* Start the DHCP service */   
   if (nIPDhcpIsUsed != 0)
   {
      /* mDNS wil be started later, if DHCP is bound */
      IP_DHCP_CallbackSet(0, DHCPCallbackBound);   
      IP_DHCP_Start(0);
   }
   
} /* EthernetInit */

/*************************************************************************/
/*  OutputBootMessage                                                    */
/*                                                                       */
/*  Output boot message.                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputBootMessage (void)
{
   const char ResetScreen[] = { 0x1B, 'c', 0 };
   
   term_printf("%s", ResetScreen);
   OS_TimeDly(50);   
   
   term_printf("\r\n");
   term_printf("*********************************\r\n");
   term_printf("  Project: %s\r\n", PROJECT_NAME);
   term_printf("  Board  : %s\r\n", TAL_BOARD);
   term_printf("  Version: v%s\r\n", PROJECT_VER_STRING);
   term_printf("  Build  : "__DATE__ " " __TIME__"\r\n");
   term_printf("*********************************\r\n");
   term_printf("\r\n");
   
} /* OutputBootMessage */

/*************************************************************************/
/*  OutputVersionInfo                                                    */
/*                                                                       */
/*  Output version information.                                          */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputVersionInfo (void)
{
   term_printf("*** Version ***\r\n");
   term_printf("Board: %s\r\n", TAL_BOARD);
   term_printf("CPU  : %s\r\n", TAL_CPU);
   term_printf("OS   : %s v%s\r\n", OS_NAME, OS_VER_STRING);
   term_printf("TAL  : v%s\r\n", TAL_CORE_VER_STRING);
   term_printf("\r\n");
   
} /* OutputVersionInfo */

/*************************************************************************/
/*  OutputFrequencyInfo                                                  */
/*                                                                       */
/*  Output frequency information.                                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputFrequencyInfo (void)
{
   term_printf("*** Frequency info ***\r\n");
   term_printf("CPU: %3d MHz\r\n", tal_CPUGetFrequencyCPU() / 1000000); 
   term_printf("AHB: %3d MHz\r\n", tal_CPUGetFrequencyAHB() / 1000000); 
   term_printf("USB: %3d MHz\r\n", tal_CPUGetFrequencyUSB() / 1000000); 
   term_printf("IPG: %3d MHz\r\n", tal_CPUGetFrequencyIPG() / 1000000); 
   term_printf("OSC: %3d MHz\r\n", tal_CPUGetFrequencyOSC() / 1000000); 
   term_printf("\r\n");
   
} /* OutputFrequencyInfo */

/*************************************************************************/
/*  OutputCPULoad                                                        */
/*                                                                       */
/*  Output load information.                                             */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OutputCPULoad (void)
{
   term_printf("CPU load: %d%%\r\n\r\n", OS_StatGetCPULoad());
   
} /* OutputCPULoad */

/*************************************************************************/
/*  OutputUsageInfo                                                      */
/*                                                                       */
/*  Output usage information.                                            */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputUsageInfo (void)
{
   term_printf("*** Usage ***\r\n");
   term_printf("h: output this info\r\n");
   term_printf("?: output this info\r\n");
   term_printf("c: output cpu load info\r\n");
   term_printf("f: output frequency info\r\n");
   term_printf("r: output runtime stack info\r\n");
   term_printf("t: output task info\r\n");
   term_printf("m: output memory info\r\n");
   term_printf("v: output version info\r\n");
   term_printf("n: output network configuration info\r\n");

#if defined(LWIP_DEBUG)
   term_printf("l: output lwIP statistics\r\n");
#endif 
   term_printf("\r\n");
   
} /* OutputUsageInfo */

/*************************************************************************/
/*  LEDTask                                                              */
/*                                                                       */
/*  This is the LED task.                                                */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void LEDTask (void *p)
{
   (void)p;
   
   while (1)
   {
      if (LED_MODE_WAIT == nLedMode)
      {
         tal_LEDSet(LED15);
         OS_TimeDly(TASK_LED_DELAY_MS);
         tal_LEDClear(LED15);
         OS_TimeDly(TASK_LED_DELAY_MS);
      }
      else
      {
         tal_LEDSet(LED15);
         OS_TimeDly(1000);
      }   
      
      //term_printf("Heap used: %d\n", tal_MEMGetUsedRawMemory());
   }

} /* LEDTask */

/*************************************************************************/
/*  StartTask                                                            */
/*                                                                       */
/*  This is the Start task.                                              */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void StartTask (void *p)
{
   (void)p;
   
   /*
    * The StartTask will be used to start all other tasks in the system.
    * At the end the priority will be set to "IDLE" priority.
    */
    
   OS_SysTickStart();   /* Start the System ticker */    
   OS_StatEnable();     /* Enable the statistic function  */

   _OSInitPost();       /* In case of uC/OS "tal_Init" will be called */

   /*******************************************************************/

   term_Start();        /* Start the Terminal functionality */
   
   /*
    * Output startup messages
    */
   OutputBootMessage();
   //OutputFrequencyInfo();
   //OutputUsageInfo();

   /* Create the LED task */
   OS_TaskCreate(&TCBLed, LEDTask, NULL, TASK_LED_PRIORITY,
                 LEDStack, sizeof(LEDStack), 
                 "LEDTask");

   EthernetInit();      /* Configure and start the IP interface */
   iperf_Start();       /* Start the IPerf service */

   OS_TaskChangePriority(TASK_START_PRIORITY_IDLE);

   while (1)
   {
      tal_LEDToggle(LED13);
      OS_TimeDly(1000);
      SecFunction();
   }

} /* StartTask */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
int main (void)
{
   /*
    * Init operating system
    */
   _OSInit(); 

   /*
    * Create the StartTask.
    * The StartTask is the one and only task which 
    * will start all other tasks of the system.
    */
   OS_TaskCreate(&TCBStartTask, StartTask, NULL, TASK_START_PRIORITY,
                 StartStack, sizeof(StartStack), 
                 "StartTask");
   
   /* 
    * OSStart must be the last function here.
    *  
    * Fasten your seatbelt, engine will be started...
    */        
   OS_Start();
   
   /*
    * This return here make no sense.
    * But to prevent the compiler warning:
    *    "return type of 'main' is not 'int'
    * We use an int as return :-)
    */ 
   return(0); /*lint !e527*/
} /* main */

/*************************************************************************/
/*  term_RxCallback                                                      */
/*                                                                       */
/*  Will be called from TermTask in case a char is received.             */
/*                                                                       */
/*  In    : bData                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void term_RxCallback (uint8_t bData)
{
   switch (bData)
   {
      case 'h':
      case '?':
      {
         OutputUsageInfo();
         break;
      } 
           
      case 'c':
      {
         OutputCPULoad();
         break;
      }

      case 'f':
      {
         OutputFrequencyInfo();
         break;
      }

      case 'r':
      {
         OS_OutputRuntimeStackInfo();
         break;
      }
      
      case 't':
      {
         OS_OutputTaskInfo();
         break;
      }
      
      case 'm':
      {
         tal_MEMOutputMemoryInfo();
         break;
      }

      case 'n':
      {
         IP_IF_OutputConfig(0);
         break;
      }  
      
      case 'v':
      {
         OutputVersionInfo();
         break;
      }

#if defined(LWIP_DEBUG)
      case 'l':
         {
         OutputLWIPStatistics();
         break;
      }
#endif       

      case 'x':
      {
         tal_CPUReboot();
         break;
      }
      
      /****************************************************/
            
      default:
      {
         /* Do nothing */
         break;
      }
   } /* end switch (bData) */
   
} /* term_RxCallback */

/*************************************************************************/
/*  HardFault_Handler                                                    */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void HardFault_Handler (void)
{

#if defined(__DEBUG__)
   while (1)
   {
      __asm__ ("nop");
   }
#endif

   tal_CPUReboot();

} /* HardFault_Handler */

/*** EOF ***/
